/*--- DOCCITT1.C --------------------- Listing 3 ---* 
 * Compute and display CCITT-CRC using a table.
 * Usage:   doccitt filename
 *
 * Author: Nigel Cort
 * (c) 1990 C Gazette. Use freely if authorship acknowledged
 *--------------------------------------------------*/
 
#include <stdio.h>

#include "ccitt.tbl"  /* created with MKCCITT.C */

void main ( int argc, char * argv[] )
{
   FILE *fin;            /* file were reading into buff         */                
   char *buff;           /* buffer we're working on             */                    
   size_t i, j;          /* counters of bytes in buff           */        
   unsigned short crc;   /* the CRC value being computed        */               

   if ( argc < 2 )
   {
      fputs ( "Error! Must specify filename.", stderr );
      exit ( 5 );
   }

   if (( fin = fopen (argv[1], "rb")) == NULL )
   {
      fprintf ( stderr, "Cannot open %s", argv[1] );
      exit ( 6 );
   }

   /*--------------------------------------------------
    * Set up a very large input buffer of 32K bytes.
    * This program does no good if it doesn't fly!
    --------------------------------------------------*/
   if (( buff = (char *) malloc (32766)) == NULL )
   {
      fputs ( " Out of memory ", stderr );
      exit ( 7 );
   }

   crc = 0;

   for (;;)
   {
      i = fread ( buff, 1, 32766, fin );
      if ( i == 0 )
      {
         if ( feof ( fin ))  /* we're done, so show results */
         {
            printf ( "CCITT CRC for %s is %04X", argv[1], crc );
            exit ( 0 );
         }
         else         /* read another 32K of file */
            continue;
      }
            
      for ( j = 0 ; j < i; j ++ )  /* loop through the buffer */
            crc = ( crc << 8 ) ^ ccitt_table [ (crc >> 8) ^ buff [j] ];
   }
}


